#define INCL_WIN
#define INCL_GPI
#define INCL_BASE
#define LINT_ARGS
 
#include <os2.h>
#include <stdlib.h>
#include <string.h>
#include "memory.h"

/****************************************************************************/

#include "memdll.h"    /* Use memory DLL downloaded from BIX */

/****************************************************************************/
 
void cdecl main(void);
MRESULT EXPENTRY MyWindowProc (HWND, USHORT, MPARAM, MPARAM);
void APIENTRY IdleLoopThread (void);
void APIENTRY MonitorLoopThread (void);
 
static USHORT   useage[MAXPOINTS+3];
static USHORT   maxpoint = MAXPOINTS;
static ULONG    counter  = 0L;
static BOOL     running  = TRUE;
static ULONG    interval = INTERVAL;
static HWND     hwndWindow;
static HWND     hwndFrame;
 
static struct INITDATA
       {
       COLOR    background;
       COLOR    line_color;
       BOOL     smoothed;
       SWP      wp;
       BOOL     slide;
       } init, saveinit;
 
static char program_name[] = "memory";

static ULONG Total_Memory;
static ULONG Free_Memory;

 
void cdecl main()
{
    UCHAR   stack1[STACKSIZE];
    UCHAR   stack2[STACKSIZE];
    HMQ     hMsgQ;
    QMSG    qMsg;
    ULONG   ctldata;
    TID     IdleTID;
    TID     MonitorTID;
    USHORT  i;
    HAB     hAB;
 
    /*  Standard PM Boilerplate:  create Anchor block and Message queue */
 
    if (!(hAB = WinInitialize (NULL)))
        exit(1);
    if (!(hMsgQ = WinCreateMsgQueue (hAB, 0)))
        exit(1);
    WinRegisterClass (hAB, "MonitorWindow", MyWindowProc, CS_SIZEREDRAW, 0);
 
    /* initialize performance data array */
 
    for (i = 0; i < MAXPOINTS; i++)
            useage[i] = 0;

    /* Get total amount of memory */
 
    Total_Memory = find_total_mem ();

    /* Get profile data saved from last time, if any */
 
    if (!WinQueryProfileData (hAB, program_name, "Monitor window data",
                              &init, &i))
        {
        /* defaults: */
        init.background = CLR_DARKCYAN;
        init.line_color = CLR_YELLOW;
        init.smoothed   = FALSE;
        init.wp.x       = 100;
        init.wp.y       = 100;
        init.wp.cx      = 220;
        init.wp.cy      = 134;
        }
 
    saveinit = init;
 
    ctldata = FCF_STANDARD & ~FCF_SHELLPOSITION;
    hwndFrame = WinCreateStdWindow (HWND_DESKTOP, 0L, &ctldata,
                                   "MonitorWindow", "", 0L, NULL,
                                   ID_MONITOR, &hwndWindow);
 
    if ((NULL == hwndFrame) || (NULL == hwndWindow))
        exit(1);
 
    WinSetWindowPos (hwndFrame, HWND_TOP,
                    init.wp.x, init.wp.y,
                    init.wp.cx, init.wp.cy,
                    SWP_SIZE | SWP_MOVE | SWP_ACTIVATE | SWP_SHOW);
 
    /* create thread to gather CPU usage data: */
 
 /* if (DosCreateThread ((void far *)IdleLoopThread, &IdleTID,
                          stack1+STACKSIZE))
        WinPostMsg (hwndWindow, WM_QUIT, 0L, 0L);
  */

 /* Set this process and all children to low priority
  */

    if (DosSetPrty (PRTYS_PROCESSTREE, PRTYC_NOCHANGE, PRTYD_MINIMUM, 0))
         exit (1);

    /* create thread to summarize Memory usage data: */
 
    if (DosCreateThread ((void far *)MonitorLoopThread, &MonitorTID,
                          stack2+STACKSIZE))
        WinPostMsg (hwndWindow, WM_QUIT, 0L, 0L);
 
    /* main message processing loop - stay here until WM_QUIT */
 
    while (WinGetMsg (hAB, (PQMSG) &qMsg, (HWND) NULL, 0, 0))
        WinDispatchMsg (hAB, (PQMSG) &qMsg);
 
    WinQueryWindowPos (hwndFrame, &init.wp);
 
    /* save position and options, if changed */
 
    if (memcmp (&init, &saveinit, sizeof init))
        WinWriteProfileData (hAB, program_name, "Monitor window data",
                            &init, sizeof init);
 
    WinDestroyWindow (hwndFrame);
    WinDestroyMsgQueue (hMsgQ);
    WinTerminate (hAB);
    DosExit (EXIT_PROCESS, 0);
}
 

VOID FormatNumber (CHAR *pchResult, ULONG ulValue)
     {
     BOOL  fDisplay = FALSE ;
     SHORT sDigit ;
     ULONG ulQuotient, ulDivisor = 1000000000L ;

     for (sDigit = 0 ; sDigit < 10 ; sDigit++)
          {
          ulQuotient = ulValue / ulDivisor ;

          if (fDisplay || ulQuotient > 0 || sDigit == 9)
               {
               fDisplay = TRUE ;

               *pchResult++ = (CHAR) ('0' + ulQuotient) ;

               if ((sDigit % 3 == 0) && sDigit != 9)
                    *pchResult++ = ',' ;
               }
          ulValue -= ulQuotient * ulDivisor ;
          ulDivisor /= 10 ;
          }
     *pchResult = '\0' ;
     }


MRESULT EXPENTRY MyWindowProc (HWND hWnd, USHORT msg, MPARAM mp1, MPARAM mp2)
{
    HPS     hPS;
    RECTL   rect;
    USHORT  shift_size;
    USHORT  loop_var;
    ULONG   xsize, ysize;
 
    static  POINTL  points[MAXPOINTS+3];
 
    CHAR    szBuffer  [24] ;
    CHAR    szBuffer2 [24] ;
    static  CHAR szTitle_Buffer [40] = "Free Memory: " ;

    switch (msg)
    {
 
    case WM_USER+NEXT_POINT:
 
        /* NEXT_POINT message is generated in the MonitorLoopThread */
 
        if (maxpoint < MAXPOINTS)
            maxpoint++;
        else if (init.slide)
                /* shift data in array */
                for (loop_var = 0; loop_var < MAXPOINTS; loop_var++)
                    useage[loop_var] = useage[loop_var+1];
        else
            {
            /* shift data in array */
            shift_size = maxpoint / 3;
            for (loop_var = 0; loop_var < MAXPOINTS - shift_size; loop_var++)
                useage[loop_var] = useage[loop_var+shift_size];
            maxpoint = MAXPOINTS - shift_size;
            }
        useage[maxpoint] = SHORT1FROMMP(mp1);
        WinInvalidateRegion (hWnd, NULL, FALSE);
        break;
 
    case WM_PAINT:
 
        hPS = WinBeginPaint (hWnd, (HPS)NULL, &rect);
 
        WinQueryWindowRect (hWnd, &rect);
 
     /* WinFillRect (hPS, &rect, init.background);  // Not needed since we draw text with DT_ERASERECT
      */
/****************************************************************************
                 Modification to add Free Memory # of Bytes
 ****************************************************************************/

        strcpy (szBuffer2, szTitle_Buffer) ;
        FormatNumber (szBuffer, Free_Memory) ;
        strcat (szBuffer2, szBuffer) ;
        strcat (szBuffer2, " bytes") ;

        WinDrawText (hPS, -1, szBuffer2, &rect,
                     CLR_NEUTRAL, CLR_BACKGROUND,
                     DT_CENTER | DT_BOTTOM | DT_ERASERECT) ;

/****************************************************************************/

        xsize = (rect.xRight - rect.xLeft + 1) * 1000 / (MAXPOINTS);
        ysize = (rect.yTop - rect.yBottom + 1) * 10;
 
        for (loop_var = 0; loop_var < maxpoint; loop_var++)
            {
            /* scale the data to fit the window */
            points[loop_var].x = loop_var * xsize / 1000;
            points[loop_var].y = useage[loop_var] * ysize / 1001;
            }
 
        GpiSetColor (hPS, init.line_color);
        if (maxpoint > 1)
            {
            GpiMove (hPS, points);
            if (init.smoothed)
                GpiPolyFillet (hPS, (ULONG) maxpoint, points);
            else
                GpiPolyLine (hPS, (ULONG) maxpoint, points);
            }

        WinEndPaint (hPS);
        break;
 
    case WM_CLOSE:
 
        running = FALSE;  /* allows the other threads to end gracefully */
        WinPostMsg(hWnd, WM_QUIT, 0L, 0L);
        break;
 
    case WM_COMMAND:
 
        switch (SHORT1FROMMP(mp1))
        {
        case IDMEXITNOW:   /* exit message from menu */
 
            WinPostMsg (hWnd, WM_CLOSE, 0L, 0L);
            break;
 
        case IDMBACKG:     /* change background color from menu */
 
            init.background++;
            if (init.background > 15)
                init.background = 0;
            break;
 
        case IDMFOREG:     /* change foreground color from menu */
 
            init.line_color++;
            if (init.line_color > 15)
                init.line_color = 0;
            break;
 
        case IDMSMOOTH:    /* toggle smooth vs. raw graph */
 
            init.smoothed = !init.smoothed;
            break;
 
        case IDMSLIDE:     /* toggle slide vs. jump */
 
            init.slide = !init.slide;
            break;
 
        default:
 
            return WinDefWindowProc (hWnd, msg, mp1, mp2);
 
        }
 
    default:
 
        return WinDefWindowProc (hWnd, msg, mp1, mp2);
 
    }
 
    return FALSE;
 
}
 
void APIENTRY IdleLoopThread ()
{
 
    /* this thread sets itself to the lowest possible priority,
       then increments a counter repeatedly */
 
    if (DosSetPrty (PRTYS_THREAD, PRTYC_IDLETIME, -31, 0))
        WinPostMsg (hwndFrame, WM_CLOSE, 0L, 0L);
    else
        while (running)
            counter++;
    DosExit (EXIT_THREAD, 0);
}
 
void APIENTRY MonitorLoopThread ()
{
    /* This thread wakes up every "interval" milliseconds,
       and collects the counter value from the IdleLoopThread.
       The data is scaled to match the actual interval, then
       send to the main thread via WinPostMsg.  */
 
    ULONG last = 0L;
    PGINFOSEG pgseg = NULL;
    PLINFOSEG plseg = NULL;
    ULONG maxcount = 0L;
    LONG   current;
    USHORT freepercent;
    ULONG last_msecs;
    ULONG current_msecs;
    ULONG delta_msecs;

 
    if (DosGetInfoSeg (&SELECTOROF(pgseg),&SELECTOROF(plseg)))
        WinPostMsg (hwndFrame, WM_CLOSE, 0L, 0L);
    else
        {
     /* last_msecs = pgseg->msecs;
      */
        while (running)
            {
            DosSleep (interval);
         /* current_msecs = pgseg->msecs;
            current = counter - last;
            last = counter;
            delta_msecs = current_msecs - last_msecs;
            last_msecs = current_msecs;
            current = (current * 1000) / delta_msecs;
            if (current > maxcount)
                maxcount = current;
            freepercent = (USHORT) ( 100L - (current * 100) / maxcount );
          */

            Total_Memory = find_total_mem ();
            Free_Memory  = find_free_mem ();

            freepercent  = (USHORT) ( 100L - ( (100L * Free_Memory) / (Total_Memory - 3000000L) ) );

            if (running)
                WinPostMsg (hwndWindow, WM_USER+NEXT_POINT,
                            MPFROMSHORT(freepercent),
                            MPFROMLONG(pgseg->time));
            }
        }
    DosExit (EXIT_THREAD, 0);
}

